# -*- python -*-
# $RCSfile: tutorialsGUI.py,v $
# $Revision: 1.32 $
# $Author: langer $
# $Date: 2005/05/19 18:16:11 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import ooferror
from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common.labeltree import LabelTree
from oof2.common import mainthread
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO import parameter
from oof2.common.IO.GUI import fileselector
from oof2.common.IO.GUI import gfxLabelTree
from oof2.common.IO.GUI import mainmenuGUI
from oof2.common.IO.GUI import subWindow
from oof2.tutorials import tutorial

from gtk import *
import re
import string


## TODO LATER: Add a table of contents.

boldtag = "BOLD("
lenboldtag = len(boldtag)
delimexpr = re.compile(r'[^\\]\)')      # finds ')' not preceded by '\'
nondelimexpr = re.compile(r'\\\)')      # finds '\)'
parasplit = re.compile(r'\n\s*\n')      # finds lines with only white space
endline = re.compile(r'\s*\n\s*')

bold = load_font("-*-fixed-*-*-*-*-*-120-*-*-*-*-iso8859-*")

class Comment:
    def __init__(self, comment, font=None):
        self.font = font
        self.commentList = []
        self.fontList = []
        # Split comment into paragraphs at blank lines.
        paragraphs = parasplit.split(comment)

        for para in paragraphs:
            # Replace newlines with spaces within paragraphs, and get
            # rid of excess white space.
            para = endline.sub(' ', para).strip()

            # Separate paragraph into strings so that each string has
            # a single font, by looking for BOLD(...).
            while para:
                index_bold = string.find(para, boldtag)
                if index_bold == -1:    # no bold text in remainder of para
                    self.commentList.append(para)
                    self.fontList.append(0)
                    break
                self.commentList.append(para[:index_bold])
                self.fontList.append(0)
                para = para[index_bold + lenboldtag:]
                # look for closing ')'
                endmatch = delimexpr.search(para)
                if not endmatch:
                    raise ooferror.ErrPyProgrammingError(
                        "Missing delimeter for BOLD tag in tutorial!")
                boldtext = para[:endmatch.start()+1]
                # replace all occurences of '\)' with ')'
                self.commentList.append(nondelimexpr.sub(')', boldtext))
                self.fontList.append(bold)
                para = para[endmatch.end():]
            self.commentList.append('\n\n')
            self.fontList.append(0)

    def isBold(self, index):
        return self.fontList[index]
    def __len__(self):
        return len(self.commentList)
    def __getitem__(self, i):
        return self.commentList[i]


####################################            

tutorialInProgress = None

def start_class(tutor, progress=0):
    debug.mainthreadTest()
    global tutorialInProgress
    if not tutorialInProgress:
        tutorialInProgress = TutorialClassGUI(tutor)
    tutorialInProgress.raise_window()
    mainmenu.OOF.Windows.Tutorial.enable()
    if progress != 0:
        tutorialInProgress.resume(progress)

tutorial.start_tutorial = start_class

# Resume a tutorial from a saved script.
def resume_tutorial(menuitem, subject, progress):
    tutor = tutorial.allTutorials[subject]
    mainthread.runBlock(start_class, (tutor, progress))
    
mainmenu.OOF.Help.Tutorials.addItem(
    oofmenu.OOFMenuItem('Resume',
                        callback=resume_tutorial,
                        secret=1,
                        params=[parameter.StringParameter('subject'),
                                parameter.IntParameter('progress')],
                        help="Resume a tutorial.  This command is only used when tutorials are saved."
                        )
    )

def raise_tutorial(menuitem):
    tutorialInProgress.raise_window()

mainmenu.OOF.Windows.addItem(oofmenu.OOFMenuItem(
    'Tutorial',
    help="Raise the tutorial window.",
    callback=raise_tutorial,
    threadable=oofmenu.UNTHREADABLE,
    gui_only=1,
    ordering=1000))
mainmenu.OOF.Windows.Tutorial.disable() # there's no window to raise, yet.
                                             


class TutorialClassGUI(subWindow.SubWindow):
    def __init__(self, tutor):
        debug.mainthreadTest()

        for menuitem in mainmenu.OOF.Help.Tutorials.items:
            menuitem.disable()

        subWindow.SubWindow.__init__(self, title=tutor.subject, menu="")

        labelhbox = GtkHBox()
        self.subject = GtkLabel()
        self.slideIndex = GtkLabel()
        labelhbox.pack_start(self.subject, TRUE, TRUE, 2)
        labelhbox.pack_end(self.slideIndex, FALSE, FALSE, 2)
        self.mainbox.pack_start(labelhbox, FALSE, FALSE, 2)

        messagebox = GtkHBox()
        self.message = GtkText()
        self.message.set_word_wrap(TRUE)
        vscroll = GtkVScrollbar(self.message.get_vadjustment())
        messagebox.add(self.message)        
        messagebox.pack_end(vscroll, FALSE, FALSE, 0)
        self.mainbox.pack_start(messagebox, TRUE, TRUE, 2)

        self.backbutton = GtkButton("Back")
        self.backbutton.connect("clicked", self.backCB)
        guitop.top().tooltip(self.backbutton, "Move to the previous slide.")
        self.nextbutton = GtkButton("Next")
        self.nextbutton.connect("clicked", self.nextCB)
        guitop.top().tooltip(self.nextbutton, "Move to the next slide.")
        self.jumpbutton = GtkButton("Jump")
        self.jumpbutton.connect("clicked", self.jumpCB)
        guitop.top().tooltip(self.jumpbutton, "Jump to the leading slide.")
        self.savebutton = GtkButton("Save")
        self.savebutton.connect("clicked", self.saveCB)
        guitop.top().tooltip(self.savebutton, "Save the progress.")
        self.closebutton = GtkButton("Close")
        self.closebutton.connect("clicked", self.closeCB)
        guitop.top().tooltip(self.closebutton, "Quit the tutorial.")
        buttonbox = GtkHBox()
        buttonbox.pack_start(self.backbutton, TRUE, TRUE, 2)
        buttonbox.pack_start(self.nextbutton, TRUE, TRUE, 2)
        buttonbox.pack_start(self.jumpbutton, TRUE, TRUE, 2)
        buttonbox.pack_start(self.savebutton, TRUE, TRUE, 2)
        buttonbox.pack_end(self.closebutton, TRUE, TRUE, 2)        
        self.mainbox.pack_end(buttonbox, FALSE, FALSE, 2)

        self.gtk.connect('destroy', self.closeCB)
        self.gtk.set_usize(500, 300)

        self.progress = 0  # How far has the tutorial gone?
                           # It's not affected by "Back" command.
        self.index = 0     # which slide?
        self.signalReceived = 0  # Received a signal, if any.
        self.tutor = tutor
        self.newLesson()
        self.tutor.lessons[0].activate()
        self.saved = None  # if saved or not

        switchboard.requestCallback("task finished", self.signalCB)

    def updateGUI(self):
        self.lesson = self.tutor.lessons[self.index]  # current lesson
        if self.lesson.subject is not None:
            self.subject.set_text(self.lesson.subject)
        else:
            self.subject.set_text("????")
        self.slideIndex.set_text("%d/%d" %
                                 ((self.index+1), len(self.tutor.lessons)))
        
        self.message.backward_delete(self.message.get_length())
        comments = Comment(self.lesson.comments)
        for i in range(len(comments)):
            comment = comments[i]
            font = comments.isBold(i)
            if font:
                self.message.insert(font, None, None, comment)
            else:
                self.message.insert_defaults(comment)
        self.message.get_vadjustment().set_value(0)
        self.sensitize()
        self.gtk.show_all()

    def newLesson(self):
        self.updateGUI()
        self.signalReceived = 0  # Resetting ...

    def sensitize(self):
        # Back, Jump, Done
        self.backbutton.set_sensitive(self.index != 0)
        self.jumpbutton.set_sensitive(self.index != self.progress)
        # Next
        self.nextbutton.set_sensitive(1)  # Default
        if self.index == self.progress:
            if self.lesson.signal and not self.signalReceived \
                   and not debug.debug():
                self.nextbutton.set_sensitive(0)
        if self.lesson == self.tutor.lessons[-1]:  # the last one?
            self.nextbutton.set_sensitive(0)

    def signalCB(self):
        if self.lesson != self.tutor.lessons[-1]:
            self.nextbutton.set_sensitive(1)

    def destroy(self):
        self.closeCB()

    def backCB(self, *args):
        self.index -= 1
        self.updateGUI()

    def nextCB(self, *args):
        if self.index == self.progress:  # move forward
            self.tutor.lessons[self.progress].deactivate()
            self.progress += 1
            self.tutor.lessons[self.progress].activate()
            self.index += 1
            self.newLesson()
        else:
            self.index += 1
            self.updateGUI()

    def jumpCB(self, *args):
        self.index = self.progress
        self.updateGUI()

    def saveCB(self, *args):
        filename = fileselector.fileSelector(ident="FileMenu",
                                             title="Save Tutorial Session")
        if filename is not None:
            file = open(filename, "w")
            mainmenu.OOF.saveLog(file)
            file.write("OOF.Help.Tutorials.Resume(subject='%s', progress=%d)\n"
                       % (self.tutor.subject, self.progress))
            file.close()
        self.saved = self.progress  # saved!

    def closeCB(self, *args):
        debug.mainthreadTest()        
        global tutorialInProgress
        if self.gtk:
            tutorialInProgress = None
            mainmenu.OOF.Windows.Tutorial.disable()
            for menuitem in mainmenu.OOF.Help.Tutorials.items:
                menuitem.enable()
            self.gtk.destroy()
            self.gtk = None

    def resume(self, where):
        self.progress = where
        self.index = self.progress
        self.updateGUI()
        self.tutor.lessons[self.progress].activate()
