# -*- python -*-
# $RCSfile: equation_.py,v $
# $Revision: 1.17 $
# $Author: langer $
# $Date: 2005/05/13 18:44:04 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from oof2.common import debug
from oof2.common import utils
from oof2.engine import component
import types

def _Equation___hash_(self):
    return self.objectid()

EquationPtr.__hash__ = _Equation___hash_

def _Equation___cmp__(self, other):
    if not isinstance(other, EquationPtr): return -1
    if self.objectid() < other.objectid(): return -1
    if self.objectid() > other.objectid(): return 1
    return 0

EquationPtr.__cmp__ = _Equation___cmp__

##EquationPtr.__repr__ = lambda self: "getEquation('%s')" % self.name()
##DivergenceEquationPtr.__repr__ = lambda self: "getEquation('%s')" % self.name()
##PlaneFluxEquationPtr.__repr__ = lambda self: "getEquation('%s')" % self.name()

EquationPtr.__repr__ = EquationPtr.name


DivergenceEquationPtr.__repr__ = DivergenceEquationPtr.name
_oldDivergenceEquation__init__ = DivergenceEquation.__init__
def DivergenceEquation__init__(self, name, flux, fields, kinetic_term=None, dynamic_term=None):
    _oldDivergenceEquation__init__(self, name, flux, fields)
    if kinetic_term is not None:
        kinetic_property = 1
        self.addTimeDerivative(kinetic_term)
        self.set_kinetic(1)
        kinetic_term.set_equation(self)
        ## add first-order time derivative
        ## to internal list of time-dependent terms
    if dynamic_term is not None:
        self.addTimeDerivative(dynamic_term)
        self.set_dynamic(1)
        ## add second-order time_derivative
        ## to internal list of time-dependent terms
        dynamic_term.set_equation(self)
        ## time dependent terms must also
        ## know their equations
    
    
DivergenceEquation.__init__ = DivergenceEquation__init__

PlaneFluxEquationPtr.__repr__ = PlaneFluxEquationPtr.name

from oof2.SWIG.engine.nodalequation import NodalEquationPtr

# AllEquations acts like a list of Equations
class _AllEquations:
    def __getitem__(self, indx):
        if type(indx) == types.IntType:
            if indx >= countEquations() or indx < 0:
                raise IndexError("Bad equation index: " + str(indx))
            return getEquationByIndex(indx)
        elif type(indx) == types.StringType:
            for i in range(countEquations()):
                eqn = getEquationByIndex(i)
                if eqn.name() == indx:
                    return eqn
            raise IndexError("Bad equation name: " + indx)
    def __len__(self):
        return countEquations()

allEquations = _AllEquations()

def getEquation(name):
    return allEquations[name]

utils.OOFdefine('getEquation', getEquation)

def _Equation___getitem__(self, comp):
    # deFortranify
    if type(comp) is types.TupleType:
        c = [x-1 for x in comp]
    else:
        c = [comp-1]
    return component.Component(self, c)

EquationPtr.__getitem__ = _Equation___getitem__


