// -*- C++ -*-
// $RCSfile: freedom.h,v $
// $Revision: 1.5 $
// $Author: redwing $
// $Date: 2005/06/08 13:21:54 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */


class DegreeOfFreedom;

#ifndef FREEDOM_H
#define FREEDOM_H

#include <iostream>
#include <string>

typedef unsigned char DOFFLAG;
const DOFFLAG ACTIVE = 1;	// fixed value
const DOFFLAG DISCONNECTED = 2; // node is irrelevant
const DOFFLAG SOLVE_SUBMESH = 4; //part of a submesh (region of a mesh) that will be solved separately

// class FreedomFlags {
// private:
//   DOFFLAG flags;
// public:
//   FreedomFlags() : flags(0) {}
//   FreedomFlags(const DOFFLAG f) : flags(f) {}
//   int disconnected() const { return flags & DISCONNECTED; }
//   void set(const DOFFLAG f) { flags |= f; }
//   void set(const FreedomFlags &fl) { set(fl.flags); }
//   void clear(const DOFFLAG f) { flags &= ~f; }
//   void clear(const FreedomFlags &fl) { clear(fl.flags); }
// };

class DegreeOfFreedom {
private:
  DegreeOfFreedom(const DegreeOfFreedom&);
public:				// should be private, but swig complains
  DegreeOfFreedom(double, int);
  ~DegreeOfFreedom() {}
  friend class FEMesh;		// only a FEMesh can create or destroy a DoF
protected:
  double value_;
  int index_;
  DOFFLAG flags;
  int nfix;			// number of times this dof has been fixed
public:
  int dofindex() const { return index_;}

  double value() const { return value_; }
  double &value() { return value_; }

//   // For slaved dof's, flags() returns the master's flags, and
//   // setflag() and clearflag() set flags in both the master and slave.
//   FreedomFlags flags() const { return flags_;}
//   FreedomFlags &flags() { return flags_; }
//   void setflag(const DOFFLAG f);
//   void setflag(const FreedomFlags &fl);
//   void setflags(const DegreeOfFreedom&);
//   virtual void clearflag(const DOFFLAG f);
//   virtual void clearflag(const FreedomFlags &fl);

  
  void fix() { nfix++; }
  void unfix() { if(nfix > 0) nfix--; }
  void free() { nfix=0; }
  bool isfree() const { return nfix == 0; }
  void activate() { flags |= ACTIVE; }
  void deactivate() { flags &= ~ACTIVE; }
  bool active() const { return flags & ACTIVE; }
  bool disconnected() const { return flags & DISCONNECTED; }

  std::string flagstr() const;

  void setsubmesh() { flags |= SOLVE_SUBMESH; }
  void unsetsubmesh() { flags &= ~SOLVE_SUBMESH; }
  bool issubmesh() const { return flags & SOLVE_SUBMESH; }
};


std::ostream &operator<<(std::ostream &os, const DegreeOfFreedom &dof);

//std::ostream &operator<<(std::ostream &os, const FreedomFlags &ff);

#endif
