// -*- C++ -*-
// $RCSfile: pixelselectioncouriere.C,v $
// $Revision: 1.2 $
// $Author: reida $
// $Date: 2004/11/30 15:16:31 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "common/cmicrostructure.h"
#include "common/coord.h"
#include "engine/cskeleton.h"
#include "engine/cskeleton.h"
#include "engine/pixelselectioncouriere.h"

ElementSelection::ElementSelection(const CMicrostructure *ms,
				   const CSkeletonElement *element)
  : PixelSelectionCourier(ms),
    element(element),
    selected(0) {}

ElementSelection::~ElementSelection() {
  delete selected;
}

void ElementSelection::start() {
  selected = element->underlying_pixels(*ms);  // get the pixel vector
  sel_iter = selected->begin();
}

ICoord ElementSelection::currentPoint() const {
  return *sel_iter;
}

void ElementSelection::next() {
  if (sel_iter == selected->end()-1 )
    done_ = true;
  else
    ++sel_iter;
}

void ElementSelection::print(std::ostream &os) const {
  os << "ElementSelection()";
}


//////////

SegmentSelection::SegmentSelection(const CMicrostructure *ms,
				   const Coord *n0, const Coord *n1)
  : PixelSelectionCourier(ms),
    n0(*n0),
    n1(*n1),
    selected(0) {}

SegmentSelection::~SegmentSelection() {
  delete selected;
}

void SegmentSelection::start() {
  selected = ms->segmentPixels(n0, n1);  // get the pixel array
  sel_iter = selected->begin();
}

ICoord SegmentSelection::currentPoint() const {
  return *sel_iter;
}

void SegmentSelection::next() {
  if(sel_iter == selected->end()-1 )
    done_ = true;
  else
    ++sel_iter;
}

void SegmentSelection::print(std::ostream &os) const {
  os << "SegmentSelection()";
}
