# -*- python -*-
# $RCSfile: preconditioner_.py,v $
# $Revision: 1.24 $
# $Author: langer $
# $Date: 2005/05/13 18:44:06 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import cregisteredclass
from oof2.common.IO import parameter
from oof2.common.IO import xmlmenudump
from types import *

cregisteredclass.registerCClass(PreconditionerPtr)

PreconditionerPtr.tip = \
                "Preconditioners for efficient solution of matrix equations."
PreconditionerPtr.discussion = xmlmenudump.loadFile(
    'DISCUSSIONS/engine/reg/preconditioner.xml')

cregisteredclass.registerCClass(UnitPreconditionerPtr)

UnitPreconditionerPtr.tip = \
          "Monolithic preconditioners for efficiently solving matrix equations."
UnitPreconditionerPtr.discussion = xmlmenudump.loadFile(
    'DISCUSSIONS/engine/reg/unitpreconditioner.xml')

# Special registration that registers the indicated class as both a
# Preconditioner and a UnitPreconditioner.
class UnitPreconditionerRegistration(cregisteredclass.Registration):
    def __init__(self, name, subclass, ordering,
                 params=[], secret=0, **kwargs):
        cregisteredclass.Registration.__init__(
            self, name, (PreconditionerPtr, UnitPreconditionerPtr),
            subclass, ordering, params, secret, **kwargs)

iluPreconditioner = UnitPreconditionerRegistration(
    "ILU",
    ILUPreconditioner,
    ordering=100,
    params=[],
    tip="Incomplete LU-factorization",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/ilu_preconditioner.xml')
    )

UnitPreconditionerRegistration(
    "IC",
    ICPreconditioner,
    ordering=200,
    params=[],
    tip="Incomplete Cholesky preconditioner. Appropriate only for meshes with symmetric, positive definite stiffness matrices.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/ic_preconditioner.xml'))

UnitPreconditionerRegistration(
    "Null",
    UnPreconditioner,
    ordering=300,
    params=[],
    tip="Be bold (or foolhardy) and attempt to solve the mesh without a preconditioner",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/null_preconditioner.xml'))

UnitPreconditionerRegistration(
    "Jacobi",
    JacobiPreconditioner,
    ordering=500,
    params=[],
    tip="A light-weight preconditioner, that simply inverts the diagonal part of the matrix.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/jacobi_preconditioner.xml'))


# Block preconditioner is composite, so it uses the ordinary
# registration process.
blockPreconditioner = cregisteredclass.Registration(
    "Block",
    PreconditionerPtr,
    BlockPreconditioner,
    ordering=1,
    params=[parameter.RegisteredParameter("inplane", UnitPreconditionerPtr,
                                          tip=parameter.emptyTipString)],
    tip="A preconditioner that treats in-plane and out-of-plane components differently.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/block_preconditioner.xml'))
