// -*- C++ -*-
// $RCSfile: permittivity.h,v $
// $Revision: 1.5 $
// $Author: langer $
// $Date: 2005/06/15 21:14:38 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

// dielectric permittivity property

#ifndef PERMITTIVITY_H
#define PERMITTIVITY_H


#include "engine/property.h"
#include "engine/matvec.h"
#include "engine/symmmatrix.h"
#include "common/pythonexportable.h"
#include <string>

class Element;
class Material;
class FEMesh;
class OrientationProp;
class ScalarField;
class OutputVal;
class PropertyOutput;
class VectorFlux;
class ElementNodeIterator;

class DielectricPermittivity : public Property {
public:
  DielectricPermittivity(PyObject *registry, const std::string &name);
  virtual void fluxmatrix(const FEMesh*, const Element*, 
			  const ElementFuncNodeIterator&,
			  const Flux*, FluxData*,
			  const MasterPosition&) const;

  virtual void cross_reference(Material*) = 0;
  virtual void output(const FEMesh*, const Element*, const PropertyOutput*,
		      const MasterPosition&, OutputVal*) const;
  virtual int K_integration_order(int) const;
  virtual int is_symmetric(const FEMesh*) const;
protected:
  MATRIX_D permittivitytensor;
  ScalarField *voltage;
  VectorFlux *total_polarization;
};

class IsoDielectricPermittivity : public DielectricPermittivity {
public:
  IsoDielectricPermittivity(PyObject *registry, 
		      const std::string &name, double epsilon);
  virtual void cross_reference(Material*) {}
  virtual void precompute();
  double epsilon() { return epsilon_; }
private:
  double epsilon_;
};

class AnisoDielectricPermittivity 
  : public DielectricPermittivity, virtual public PythonNative<Property> 
{
public:
  AnisoDielectricPermittivity(PyObject *registry, PyObject *self, 
			    const std::string &name,
			    SymmMatrix3 *epsilon);
  virtual void cross_reference(Material*); // finds Orientation
  virtual void precompute();
  SymmMatrix3 *epsilon() { return &epsilon_; }
private:
  SymmMatrix3 epsilon_;
  OrientationProp *orientation;
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// distributed heat source

class ChargeDensity : public Property {
private:
  double qdot_; // external heat dumped in per unit area per unit time.
  VectorFlux *total_polarization;
public:
  ChargeDensity(PyObject *registry, const std::string &name, double qd);
  double rho() { return qdot_; }
  virtual int K_integration_order(int) const;
  virtual void fluxrhs(const FEMesh*, const Element*, 
		       const Flux*, FluxData*, const MasterPosition&) const;
   virtual int is_symmetric(const FEMesh* mesh) const {return 1;}
};

#endif
